==> packager-10.2.0-623.113-sources.jar: com/xebialabs/xldeploy/packager/MustacheScanningReader.scala <==
/**
 * Copyright 2014-2019 XebiaLabs Inc. and its affiliates. Use is subject to terms of the enclosed Legal Notice.
 */
package com.xebialabs.xldeploy.packager

import java.io.Reader

import com.xebialabs.xldeploy.packager.Delims._
import grizzled.slf4j.Logging

class MustacheScanningReader(mustacher: Mustacher, wrapped: Reader) extends Reader {
  override def read(chars: Array[Char], off: Int, len: Int): Int = {
    val readCount = wrapped.read(chars, off, len)
    if (readCount > 0) {
      mustacher.handle(chars, off, readCount)
    }
    readCount
  }

  override def close(): Unit = {
    wrapped.close()
    mustacher.resetState()
  }
}

object Mustacher extends MustacheReader {
  def apply(delims: String): Mustacher = new Mustacher(new Delims(delims))
}

class Mustacher(delims: Delims) extends Logging {
  import com.xebialabs.xldeploy.packager.Mustacher._

  var placeholders: Set[String] = Set.empty
  var state: Mustacher.State = Mustacher.Text
  var lookBehind: Char = _
  val invalids: Set[Char] = Set('\r', '\n')


  def newReader(reader: Reader): MustacheScanningReader = new MustacheScanningReader(this, reader)

  def handle(contents: Array[Char], offset: Int, len: Int): Unit = {
    for (i <- 0 until len) {
      handle(contents(offset + i))
    }
  }

  def addPlaceholder(tag: Array[Char]): Unit = {
    val stringTag = tag.mkString.trim
    logger.debug(s"Found placeholder: [$stringTag]")

    if (!stringTag.isEmpty) {
      placeholders += stringTag
    }
  }

  def handle(c: Char): Unit = {
    state match {
      case Text if delims.matchesStart(c) == Partial => // 2-char start delimiter
        state = MatchStartTag
      case Text if delims.matchesStart(c) == Full => // 1-char start delimiter
        state = MatchingTag(Array())
      case Text => // Ok
      case MatchStartTag if delims.matchesStart(lookBehind, c) == Full => // 2-char start delimiter
        state = MatchingTag(Array())
      case MatchStartTag => // Not matched delimiter, continue in text mode
        state = Text
      case MatchingTag(tag) if delims.matchesEnd(c) == Full => // 1-char end delimiter
        addPlaceholder(tag)
        state = Text
      case MatchingTag(cs) if delims.matchesEnd(c) == Partial => // 2-char end delimiter
        state = MatchEndTag(cs)
      case MatchingTag(cs) if delims.matchesStart(c) == Partial => // 2-char nested delimiter start with partial match for previously found
        state = MatchNestedStart(cs)
      case MatchingTag(_) if delims.matchesStart(c) == Full => // 2-char nested delimiter start with full match for previously found and possibly some characters scanned
        state = MatchingTag(Array()) // Throw away currently matched tag
      case MatchingTag(cs) if invalids.contains(c) =>
        // No support for multiline placeholders, stop processing
        logger.debug(s"Found and skipping invalid placeholder with linefeed and/or carriage return character: [${cs.mkString.trim}]")
        state = Text
      case MatchingTag(cs) =>
        state = MatchingTag(cs :+ c)
      case MatchNestedStart(_) if delims.matchesStart(lookBehind, c) == Full => // 2-char nested delimiter start
        state = MatchingTag(Array()) // Throw away currently matched tag
      case MatchNestedStart(cs) => // Not matched full nested start-delimiter, continue tag-matching
        state = MatchingTag(cs :+ c)
      case MatchEndTag(tag) if delims.matchesEnd(lookBehind, c) == Full => // 2-char end delimiter
        addPlaceholder(tag)
        state = Text
      case MatchEndTag(tag) => // Not matched full end-delimiter, continue tag-matching
        state = MatchingTag(tag :+ lookBehind :+ c)
    }
    lookBehind = c
  }

  def resetState(): Unit = {
    state = Text
  }
}

==> packager-25.3.0-915.1055-sources.jar: com/xebialabs/xldeploy/packager/MustacheScanningReader.scala <==
/**
 * Copyright 2014-2019 XebiaLabs Inc. and its affiliates. Use is subject to terms of the enclosed Legal Notice.
 */
package com.xebialabs.xldeploy.packager

import java.io.Reader
import com.xebialabs.xldeploy.packager.Delims._
import grizzled.slf4j.Logger
import org.slf4j.LoggerFactory

class MustacheScanningReader(mustacher: Mustacher, wrapped: Reader) extends Reader {
  override def read(chars: Array[Char], off: Int, len: Int): Int = {
    val readCount = wrapped.read(chars, off, len)
    if (readCount > 0) {
      mustacher.handle(chars, off, readCount)
    }
    readCount
  }

  override def close(): Unit = {
    wrapped.close()
    mustacher.resetState()
  }
}

object Mustacher extends MustacheReader {

  private[Mustacher] val logger = new Logger(LoggerFactory.getLogger(classOf[Mustacher]))

  def apply(delims: String): Mustacher = new Mustacher(new Delims(delims))

  val invalids: Set[Char] = Set('\r', '\n')
}

class Mustacher(delims: Delims) {
  import com.xebialabs.xldeploy.packager.Mustacher._

  var placeholders: Set[String] = Set.empty
  var state: Mustacher.State = Mustacher.Text
  var lookBehind: Char = _

  def newReader(reader: Reader): MustacheScanningReader = new MustacheScanningReader(this, reader)

  def handle(contents: Array[Char], offset: Int, len: Int): Unit = {
    for (i <- 0 until len) {
      handle(contents(offset + i))
    }
  }

  def addPlaceholder(tag: Array[Char]): Unit = {
    val stringTag = tag.mkString.trim
    logger.debug(s"Found placeholder: [$stringTag]")

    if (stringTag.nonEmpty) {
      placeholders += stringTag
    }
  }

  def handle(c: Char): Unit = {
    state match {
      case Text if delims.matchesStart(c) == Partial => // 2-char start delimiter
        state = MatchStartTag
      case Text if delims.matchesStart(c) == Full => // 1-char start delimiter
        state = matchingTagEmptyArray
      case Text => // Ok
      case MatchStartTag if delims.matchesStart(lookBehind, c) == Full => // 2-char start delimiter
        state = matchingTagEmptyArray
      case MatchStartTag => // Not matched delimiter, continue in text mode
        state = Text
      case MatchingTag(tag) if delims.matchesEnd(c) == Full => // 1-char end delimiter
        addPlaceholder(tag)
        state = Text
      case MatchingTag(cs) if delims.matchesEnd(c) == Partial => // 2-char end delimiter
        state = MatchEndTag(cs)
      case MatchingTag(cs) if delims.matchesStart(c) == Partial => // 2-char nested delimiter start with partial match for previously found
        state = MatchNestedStart(cs)
      case MatchingTag(_) if delims.matchesStart(c) == Full => // 2-char nested delimiter start with full match for previously found and possibly some characters scanned
        state = matchingTagEmptyArray // Throw away currently matched tag
      case MatchingTag(cs) if invalids.contains(c) =>
        // No support for multiline placeholders, stop processing
        logger.debug(s"Found and skipping invalid placeholder with linefeed and/or carriage return character: [${cs.mkString.trim}]")
        state = Text
      case MatchingTag(cs) =>
        state = MatchingTag(cs :+ c)
      case MatchNestedStart(_) if delims.matchesStart(lookBehind, c) == Full => // 2-char nested delimiter start
        state = matchingTagEmptyArray // Throw away currently matched tag
      case MatchNestedStart(cs) => // Not matched full nested start-delimiter, continue tag-matching
        state = MatchingTag(cs :+ c)
      case MatchEndTag(tag) if delims.matchesEnd(lookBehind, c) == Full => // 2-char end delimiter
        addPlaceholder(tag)
        state = Text
      case MatchEndTag(tag) => // Not matched full end-delimiter, continue tag-matching
        state = MatchingTag(tag :+ lookBehind :+ c)
    }
    lookBehind = c
  }

  def resetState(): Unit = {
    state = Text
  }
}
